@@ -34,6 +34,8 @@ func AddToScheme(scheme *runtime.Scheme) error {
 		&DevSessionInstanceList{},
 		&ProjectInstance{},
 		&ProjectInstanceList{},
+		&ImageMetadataCache{},
+		&ImageMetadataCacheList{},
 	)
 
 	// Add common types
