@@ -2,6 +2,7 @@ package appdefinition
 
 import (
 	"crypto/sha256"
+	_ "embed"
 	"encoding/hex"
 	"encoding/json"
 	"errors"
@@ -24,6 +25,7 @@ import (
 	"github.com/acorn-io/runtime/pkg/publicname"
 	"github.com/acorn-io/runtime/pkg/ref"
 	"github.com/acorn-io/runtime/pkg/secrets"
+	"github.com/acorn-io/runtime/pkg/services"
 	"github.com/acorn-io/runtime/pkg/system"
 	"github.com/acorn-io/runtime/pkg/volume"
 	"github.com/acorn-io/z"
@@ -60,6 +62,7 @@ func DeploySpec(req router.Request, resp router.Response) (err error) {
 	appInstance := req.Object.(*v1.AppInstance)
 	status := condition.Setter(appInstance, resp, v1.AppInstanceConditionDefined)
 	interpolator := secrets.NewInterpolator(req.Ctx, req.Client, appInstance)
+	var result []kclient.Object
 
 	defer func() {
 		if err == nil {
@@ -89,39 +92,44 @@ func DeploySpec(req router.Request, resp router.Response) (err error) {
 		return err
 	}
 
-	if err := addDeployments(req, appInstance, tag, pullSecrets, interpolator, resp); err != nil {
+	if objs, err := ToDeployments(req, appInstance, tag, pullSecrets, interpolator); err != nil {
 		return err
+	} else if len(objs) > 0 {
+		result = append(result, objs...)
 	}
-	if err := addRouters(appInstance, resp); err != nil {
+	if objs, err := toRouters(appInstance); err != nil {
 		return err
+	} else {
+		result = append(result, objs...)
 	}
-	if err := addJobs(req, appInstance, tag, pullSecrets, interpolator, resp); err != nil {
+	if objs, err := toJobs(req, appInstance, pullSecrets, tag, interpolator); err != nil {
 		return err
+	} else {
+		result = append(result, objs...)
 	}
-	if err := addServices(req, appInstance, interpolator, resp); err != nil {
+	if objs, err := services.ToAcornServices(req.Ctx, req.Client, interpolator, appInstance); err != nil {
 		return err
+	} else {
+		result = append(result, objs...)
 	}
-	if err := addPVCs(req, appInstance, resp); err != nil {
+	if objs, err := toPVCs(req, appInstance); err != nil {
 		return err
+	} else {
+		result = append(result, objs...)
 	}
-	if err := addAcorns(req, appInstance, tag, pullSecrets, resp); err != nil {
+	if objs, err := toAcorns(req, appInstance, tag, pullSecrets); err != nil {
 		return err
+	} else {
+		result = append(result, objs...)
 	}
 
+	// Secrets go in first so that they are created/updated before things that depend on them.
 	resp.Objects(pullSecrets.Objects()...)
 	resp.Objects(interpolator.Objects()...)
+	resp.Objects(result...)
 	return pullSecrets.Err()
 }
 
-func addDeployments(req router.Request, appInstance *v1.AppInstance, tag name.Reference, pullSecrets *PullSecrets, secrets *secrets.Interpolator, resp router.Response) error {
-	deps, err := ToDeployments(req, appInstance, tag, pullSecrets, secrets)
-	if err != nil {
-		return err
-	}
-	resp.Objects(deps...)
-	return nil
-}
-
 func toEnvFrom(envs []v1.EnvVar) (result []corev1.EnvFromSource) {
 	for _, env := range envs {
 		if env.Secret.Name != "" && env.Secret.Key == "" {
@@ -205,7 +213,7 @@ func hasContextDir(container v1.Container) bool {
 	return false
 }
 
-func toContainers(app *v1.AppInstance, tag name.Reference, name string, container v1.Container, interpolator *secrets.Interpolator) ([]corev1.Container, []corev1.Container) {
+func toContainers(app *v1.AppInstance, tag name.Reference, name string, container v1.Container, interpolator *secrets.Interpolator, addWait bool) ([]corev1.Container, []corev1.Container) {
 	var (
 		containers     []corev1.Container
 		initContainers []corev1.Container
@@ -226,10 +234,10 @@ func toContainers(app *v1.AppInstance, tag name.Reference, name string, containe
 		})
 	}
 
-	newContainer := toContainer(app, tag, name, container, interpolator)
+	newContainer := toContainer(app, tag, name, container, interpolator, addWait && len(container.Ports) > 0)
 	containers = append(containers, newContainer)
 	for _, entry := range typed.Sorted(container.Sidecars) {
-		newContainer = toContainer(app, tag, entry.Key, entry.Value, interpolator)
+		newContainer = toContainer(app, tag, entry.Key, entry.Value, interpolator, addWait && len(entry.Value.Ports) > 0)
 
 		if entry.Value.Init {
 			initContainers = append(initContainers, newContainer)
@@ -254,7 +262,7 @@ func sanitizeVolumeName(name string) string {
 	return name
 }
 
-func toMounts(app *v1.AppInstance, container v1.Container, interpolation *secrets.Interpolator) (result []corev1.VolumeMount) {
+func toMounts(app *v1.AppInstance, container v1.Container, interpolation *secrets.Interpolator, addWait bool) (result []corev1.VolumeMount) {
 	for _, entry := range typed.Sorted(container.Files) {
 		suffix := ""
 		if volume.NormalizeMode(entry.Value.Mode) != "" {
@@ -297,6 +305,13 @@ func toMounts(app *v1.AppInstance, container v1.Container, interpolation *secret
 			})
 		}
 	}
+
+	if addWait {
+		result = append(result, corev1.VolumeMount{
+			Name:      string(app.UID),
+			MountPath: AcornHelperSleepPath,
+		})
+	}
 	return
 }
 
@@ -420,7 +435,7 @@ func toProbe(container v1.Container, probeType v1.ProbeType) *corev1.Probe {
 	return nil
 }
 
-func toContainer(app *v1.AppInstance, tag name.Reference, containerName string, container v1.Container, interpolator *secrets.Interpolator) corev1.Container {
+func toContainer(app *v1.AppInstance, tag name.Reference, containerName string, container v1.Container, interpolator *secrets.Interpolator, addWait bool) corev1.Container {
 	containerObject := corev1.Container{
 		Name:           containerName,
 		Image:          images.ResolveTag(tag, container.Image),
@@ -432,13 +447,26 @@ func toContainer(app *v1.AppInstance, tag name.Reference, containerName string,
 		TTY:            container.Interactive,
 		Stdin:          container.Interactive,
 		Ports:          toPorts(container),
-		VolumeMounts:   toMounts(app, container, interpolator),
+		VolumeMounts:   toMounts(app, container, interpolator, addWait),
 		LivenessProbe:  toProbe(container, v1.LivenessProbeType),
 		StartupProbe:   toProbe(container, v1.StartupProbeType),
 		ReadinessProbe: toProbe(container, v1.ReadinessProbeType),
 		Resources:      app.Status.Scheduling[containerName].Requirements,
 	}
 
+	if addWait {
+		// If a container exposes a port, then add a pre-stop lifecycle hook that sleeps for 5 seconds. This should allow the
+		// endpoints controller to remove the pods IP on termination and stop sending traffic to the container.
+		// Note that this requires the acorn-sleep binary to be in the container, via the volume mount.
+		containerObject.Lifecycle = &corev1.Lifecycle{
+			PreStop: &corev1.LifecycleHandler{
+				Exec: &corev1.ExecAction{
+					Command: []string{fmt.Sprintf("%s/%s", AcornHelperSleepPath, app.UID)},
+				},
+			},
+		}
+	}
+
 	return containerObject
 }
 
@@ -636,18 +664,19 @@ func getSecretAnnotations(req router.Request, appInstance *v1.AppInstance, conta
 func toDeployment(req router.Request, appInstance *v1.AppInstance, tag name.Reference, name string, container v1.Container, pullSecrets *PullSecrets, interpolator *secrets.Interpolator) (*appsv1.Deployment, error) {
 	var (
 		stateful = isStateful(appInstance, container)
+		addWait  = !stateful && len(acornSleepBinary) > 0 && z.Dereference(container.Scale) > 1 && !appInstance.Status.GetDevMode()
 	)
 
 	interpolator = interpolator.ForContainer(name)
 
-	containers, initContainers := toContainers(appInstance, tag, name, container, interpolator)
+	containers, initContainers := toContainers(appInstance, tag, name, container, interpolator, addWait)
 
 	secretAnnotations, err := getSecretAnnotations(req, appInstance, container, interpolator)
 	if err != nil {
 		return nil, err
 	}
 
-	volumes, err := toVolumes(appInstance, container, interpolator)
+	volumes, err := toVolumes(appInstance, container, interpolator, addWait)
 	if err != nil {
 		return nil, err
 	}
@@ -685,7 +714,7 @@ func toDeployment(req router.Request, appInstance *v1.AppInstance, tag name.Refe
 					Affinity:                      appInstance.Status.Scheduling[name].Affinity,
 					Tolerations:                   appInstance.Status.Scheduling[name].Tolerations,
 					PriorityClassName:             appInstance.Status.Scheduling[name].PriorityClassName,
-					TerminationGracePeriodSeconds: z.Pointer[int64](5),
+					TerminationGracePeriodSeconds: z.Pointer[int64](10),
 					ImagePullSecrets:              pullSecrets.ForContainer(name, append(containers, initContainers...)),
 					EnableServiceLinks:            new(bool),
 					Containers:                    containers,
@@ -720,6 +749,7 @@ func toDeployment(req router.Request, appInstance *v1.AppInstance, tag name.Refe
 }
 
 func ToDeployments(req router.Request, appInstance *v1.AppInstance, tag name.Reference, pullSecrets *PullSecrets, secrets *secrets.Interpolator) (result []kclient.Object, _ error) {
+	var configMapInitialized bool
 	for _, containerName := range typed.SortedKeys(appInstance.Status.AppSpec.Containers) {
 		containerDef := appInstance.Status.AppSpec.Containers[containerName]
 		if ports.IsLinked(appInstance, containerName) {
@@ -733,6 +763,29 @@ func ToDeployments(req router.Request, appInstance *v1.AppInstance, tag name.Ref
 		if err != nil {
 			return nil, err
 		}
+
+		if !configMapInitialized {
+			// This configmap only needs to be created once for the whole app instance.
+			for _, v := range dep.Spec.Template.Spec.Volumes {
+				if v.Name == string(appInstance.UID) {
+					configMapInitialized = true
+					result = append(result, &corev1.ConfigMap{
+						ObjectMeta: metav1.ObjectMeta{
+							Name:      string(appInstance.UID),
+							Namespace: appInstance.Status.Namespace,
+							Labels: map[string]string{
+								labels.AcornManaged: "true",
+							},
+						},
+						BinaryData: map[string][]byte{
+							string(appInstance.UID): acornSleepBinary,
+						},
+					})
+					break
+				}
+			}
+		}
+
 		perms := v1.FindPermission(containerName, appInstance.Status.Permissions)
 		sa, err := toServiceAccount(req, dep.GetName(), dep.GetLabels(), dep.GetAnnotations(), appInstance, perms)
 		if err != nil {
@@ -747,5 +800,6 @@ func ToDeployments(req router.Request, appInstance *v1.AppInstance, tag name.Ref
 		}
 		result = append(result, sa, dep, pdb.ToPodDisruptionBudget(dep))
 	}
+
 	return result, nil
 }
