@@ -68,8 +68,12 @@ func registryDeployment(namespace, serviceAccountName, registryImage string, req
 					},
 				},
 				Spec: corev1.PodSpec{
-					PriorityClassName:  system.AcornPriorityClass,
-					EnableServiceLinks: new(bool),
+					SecurityContext: &corev1.PodSecurityContext{
+						FSGroup: z.Pointer[int64](1000),
+					},
+					TerminationGracePeriodSeconds: z.Pointer[int64](10),
+					PriorityClassName:             system.AcornPriorityClass,
+					EnableServiceLinks:            new(bool),
 					Containers: []corev1.Container{
 						{
 							Name: "registry",
@@ -122,6 +126,17 @@ func registryDeployment(namespace, serviceAccountName, registryImage string, req
 									MountPath: "/var/lib/registry",
 								},
 							},
+							Lifecycle: &corev1.Lifecycle{
+								PreStop: &corev1.LifecycleHandler{
+									Exec: &corev1.ExecAction{
+										Command: []string{
+											"/bin/sh",
+											"-c",
+											"sleep 5",
+										},
+									},
+								},
+							},
 						},
 					},
 					ServiceAccountName: serviceAccountName,
